package com.perforce.hws.plugin.perforce.routes;

import com.perforce.hws.core.p4base.P4Methods;
import com.perforce.hws.core.p4base.ResultMap;
import com.perforce.hws.core.p4base.ResultMapMethods;
import com.perforce.hws.core.sessions.SessionData;
import com.perforce.hws.route.HasArgList;
import com.perforce.hws.route.UsesCommandWhitelist;
import com.perforce.hws.utils.HWSSettings;
import com.perforce.hws.utils.ModelConversion;
import com.perforce.hws.utils.UsesServerHandles;

import static spark.Spark.halt;

/**
 * GET /p4/:api/commands/:command
 */
public class PerforceRunCommandRoute implements
        spark.Route,
        HasArgList,
        UsesCommandWhitelist,
        P4Methods,
        ResultMapMethods,
        ModelConversion,
        UsesServerHandles {

    @Override
    public Object handle(spark.Request request, spark.Response response) throws Exception {
        SessionData sessionData = request.attribute("sessionData");
        HWSSettings settings = request.attribute("settings");

        String server = request.params("server");
        String command = request.params("command");

        java.util.List<String> argList = argList(request);

        if (!isWhitelisted(settings, command, argList)) {
            halt(400);
        }

        return withServerHandle(server, sessionData, settings, serverHandle -> {
            java.util.List<ResultMap> resultMaps =
                    exec(serverHandle, command, argList.toArray(new String[0]));

            return toCommandResponse(resultMaps);
        });
    }
}
