package com.perforce.hws.plugin.perforce.routes;

import com.perforce.hws.core.p4base.HasGson;
import com.perforce.hws.core.p4base.P4Methods;
import com.perforce.hws.core.p4base.ResultMap;
import com.perforce.hws.core.p4base.ResultMapMethods;
import com.perforce.hws.core.sessions.SessionData;
import com.perforce.hws.route.UsesJsonRequests;
import com.perforce.hws.utils.HWSSettings;
import com.perforce.hws.utils.ModelConversion;
import com.perforce.hws.utils.UsesServerHandles;
import com.perforce.p4client.models.JobCommand;

/**
 * PATCH /{server}/jobs/{job}
 */
public class PerforceUpdateJobRoute implements
        spark.Route,
		ModelConversion,
		P4Methods,
		ResultMapMethods,
		HasGson,
		UsesJsonRequests,
		UsesServerHandles {

    @Override
    public Object handle(spark.Request request, spark.Response response) throws Exception {
        SessionData sessionData = request.attribute("sessionData");
        HWSSettings settings = request.attribute("settings");

        String server = request.params("server");
        String job = request.params("job");

        JobCommand jobCommand = readModel(request, JobCommand.class);
        java.util.Map<String, Object> updates = toRequestMap(jobCommand);

        return withServerHandle(server, sessionData, settings, serverHandle -> {

            java.util.List<ResultMap> results = exec(serverHandle, "job", "-o", job);
            ResultMap resultMap = firstResultOrFail(results);
            java.util.Map<String, Object> saved = toRequestMap(toJobCommand(resultMap));

            saved.putAll(updates);

            // P4Java adds keys that sometimes are illegal... on some commands.
            if (saved.containsKey("specFormatted")) {
                saved.remove("specFormatted");
            }

            return toCommandResponse(execInput(serverHandle, saved, "job", "-i"));
        });
    }
}

