package com.perforce.hws.plugin.perforce;

import com.google.gson.Gson;
import com.perforce.hws.HelixWebServices;
import com.perforce.hws.core.test.EmptySparkRule;
import com.perforce.hws.core.test.SimpleResponse;
import com.perforce.hws.core.test.SimpleTestSpark;
import com.perforce.hws.filters.PlatformVersionFilter;
import com.perforce.hws.filters.SettingsFilter;
import com.perforce.hws.filters.VersionFilter;
import com.perforce.hws.utils.HWSSettings;
import com.perforce.hws.utils.P4dConfig;
import com.perforce.p4client.models.LoginResponse;
import org.junit.Rule;
import spark.Service;

import java.io.File;
import java.net.URL;

/**
 * Created by pallen on 15/08/2016.
 */
public abstract class DefaultFunctionalTests {

	protected static final String P4BIN = "../../core/src/test/resources/";
	protected static final String ROOT = "build/rsh_root";
	protected static final String VERSION = "16.1";
	protected static final int PORT = 9000;
	protected static final File PLUGIN = new File("build/libs/PerforcePlugin-0.1.0.zip");
	protected static final String ID = "testID";
	protected static final String HOST = "http://localhost:";

	@Rule
	public EmptySparkRule spark = new EmptySparkRule(PORT, PLUGIN);

	public P4dConfig p4dConfig = new P4dConfig();

	public void init(String port) throws Exception {
		p4dConfig.setId(ID);
		p4dConfig.setP4PORT(port);

		HWSSettings hwsSettings = new HWSSettings();
		hwsSettings.addP4dConfigMap(p4dConfig);
		hwsSettings.setJwtSigningKey("MheXr1H5GGLvLpLDbQ4pofTAHN6ISUG7wBNRA89926ar9rZYfB5bnqPzi90E");

		// Load filters
		Service service = spark.getService();
		service.before(new VersionFilter());
		service.before(new SettingsFilter(hwsSettings));
		service.before(new PlatformVersionFilter(hwsSettings));
		HelixWebServices.addExceptionHandlers(service);
	}

	public String login(String user, String pass) throws Exception {
		URL loginUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/login");
		SimpleResponse response = SimpleTestSpark.login(loginUrl, user, pass);

		Gson gson = new Gson();
		LoginResponse login = gson.fromJson(response.body, LoginResponse.class);
		return login.getTicket();
	}
}
