package com.perforce.hws.plugin.perforce;

import com.google.gson.Gson;
import com.perforce.hws.core.test.EmptyServerRule;
import com.perforce.hws.core.test.SimpleResponse;
import com.perforce.hws.core.test.SimpleTestSpark;
import com.perforce.p4client.models.LoginResponse;
import com.perforce.p4java.impl.generic.core.User;
import com.perforce.p4java.server.IServer;
import com.perforce.p4java.server.ServerFactory;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;

import java.io.IOException;
import java.net.URL;

import static org.hamcrest.CoreMatchers.startsWith;

/**
 * Created by pallen on 11/08/2016.
 */
public class LoginFunctionalTests extends DefaultFunctionalTests {

	@Rule
	public EmptyServerRule p4d = new EmptyServerRule(P4BIN, ROOT, VERSION);

	@Before
	public void before() throws Exception {
		init(p4d.getRshPort());
	}

	@Test
	public void testLogin() throws Exception {

		IServer server = ServerFactory.getServer(p4dConfig.toURIString(), null);
		server.connect();
		server.createUser(User.newUser("admin", "admin@email", "Admin", "Password"), true);
		server.disconnect();

		// Test login with 'admin' user
		URL loginUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/login");
		SimpleResponse response = SimpleTestSpark.login(loginUrl, "admin", "Password");

		Assert.assertEquals("Code", 200, response.status);
		Assert.assertTrue("Ticket", response.body.contains("ticket"));

		Gson gson = new Gson();
		LoginResponse login = gson.fromJson(response.body, LoginResponse.class);
		Assert.assertTrue("GSON", !login.getTicket().isEmpty());

		// Test 'Users' with ticket.
		URL usersUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users");

		// ... fail as no auth ticket passed
		try {
			response = SimpleTestSpark.get(usersUrl, "");
		} catch (IOException e) {
			Assert.assertThat(e.getMessage(), startsWith("Server returned HTTP response code: 403"));
		}

		response = SimpleTestSpark.get(usersUrl, login.getTicket());
		Assert.assertEquals("Code", 200, response.status);
	}
}
