package com.perforce.hws.plugin.perforce;

import com.google.gson.Gson;
import com.perforce.hws.core.test.EmptyServerRule;
import com.perforce.hws.core.test.SimpleResponse;
import com.perforce.hws.core.test.SimpleTestSpark;
import com.perforce.p4client.models.UserCommand;
import com.perforce.p4client.models.UsersCommand;
import com.perforce.p4java.core.IUserSummary;
import com.perforce.p4java.impl.generic.core.User;
import com.perforce.p4java.server.IServer;
import com.perforce.p4java.server.ServerFactory;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;

import java.net.URL;

/**
 * Created by pallen on 15/08/2016.
 */
public class UserFunctionalTests extends DefaultFunctionalTests {

	@Rule
	public EmptyServerRule p4d = new EmptyServerRule(P4BIN, ROOT, VERSION);

	@Before
	public void before() throws Exception {
		init(p4d.getRshPort());
	}

	@Test
	public void testUser() throws Exception {

		IServer server = ServerFactory.getServer(p4dConfig.toURIString(), null);
		server.connect();
		server.createUser(User.newUser("bob", "bob@email", "Bob", "Password"), true);
		server.createUser(new User("joe", "joe@email", "Joe", null, null, "Password", null,
				IUserSummary.UserType.OPERATOR, null), true);
		server.createUser(User.newUser("admin", "admin@email", "Admin", "Password"), true);
		server.disconnect();

		String ticket = login("admin", "Password");

		// Test GET /users
		URL usersUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users?includeService=true");
		SimpleResponse response = SimpleTestSpark.get(usersUrl, ticket);
		Assert.assertEquals("Code", 200, response.status);

		Gson gson = new Gson();
		UsersCommand[] users = gson.fromJson(response.body, UsersCommand[].class);
		Assert.assertEquals("Users", 4, users.length);

		// Test GET /users/bob
		URL userUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users/bob");
		response = SimpleTestSpark.get(userUrl, ticket);
		Assert.assertEquals("Code", 200, response.status);

		gson = new Gson();
		UserCommand user = gson.fromJson(response.body, UserCommand.class);
		Assert.assertEquals("User", "bob", user.getUser());
		Assert.assertEquals("Name", "Bob", user.getFullName());
		Assert.assertEquals("Email", "bob@email", user.getEmail());
		Assert.assertEquals("Type", "standard", user.getType());

		// Test GET /users/joe (service user)
		userUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users/joe");
		response = SimpleTestSpark.get(userUrl, ticket);
		Assert.assertEquals("Code", 200, response.status);

		gson = new Gson();
		user = gson.fromJson(response.body, UserCommand.class);
		Assert.assertEquals("User", "joe", user.getUser());
		Assert.assertEquals("Type", "operator", user.getType());

		// Create user reb (service) -- Test POST /users
		user = new UserCommand();
		user.setUser("reb");
		user.setEmail("reb@email");
		user.setPassword("Password");
		user.setFullName("Reb");
		user.setType("service");
		user.setAuthMethod("perforce");

		String json = gson.toJson(user);
		userUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users");
		response = SimpleTestSpark.post(userUrl, ticket, json);
		Assert.assertEquals("Code", 200, response.status);

		// Check user count
		usersUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users?includeService=true");
		response = SimpleTestSpark.get(usersUrl, ticket);
		users = gson.fromJson(response.body, UsersCommand[].class);
		Assert.assertEquals("Users", 5, users.length);

		// Update email -- Test PATCH /users/reb
		user.setEmail("reb@foo");
		json = gson.toJson(user);

		userUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users/reb");
		response = SimpleTestSpark.patch(userUrl, ticket, json);
		Assert.assertEquals("Code", 200, response.status);

		// Test GET /users/reb (service user)
		userUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users/reb");
		response = SimpleTestSpark.get(userUrl, ticket);
		Assert.assertEquals("Code", 200, response.status);

		gson = new Gson();
		user = gson.fromJson(response.body, UserCommand.class);
		Assert.assertEquals("User", "reb", user.getUser());
		Assert.assertEquals("Email", "reb@foo", user.getEmail());

		// Delete bob
		userUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users/bob");
		response = SimpleTestSpark.delete(userUrl, ticket);
		Assert.assertEquals("Code", 200, response.status);

		// Check user count (normal users: nouser and admin)
		usersUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users");
		response = SimpleTestSpark.get(usersUrl, ticket);
		users = gson.fromJson(response.body, UsersCommand[].class);
		Assert.assertEquals("Users", 2, users.length);

		// Check max limit
		usersUrl = new URL(HOST + PORT + "/api/p4/v" + VERSION + "/" + ID + "/users?max=1");
		response = SimpleTestSpark.get(usersUrl, ticket);
		users = gson.fromJson(response.body, UsersCommand[].class);
		Assert.assertEquals("Users", 1, users.length);
	}
}
